package de.arvato.systems.nmvs.example;

import static java.util.Comparator.comparing;

import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Scanner;

import javax.xml.ws.soap.SOAPBinding;

import org.apache.cxf.frontend.ClientProxyFactoryBean;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import org.apache.cxf.jaxws.JaxWsProxyFactoryBean;

import de.arvato.systems.nmvs.example.v3.WsSinglePackExample;
import eu.nmvs.services.v3_0.ISinglePackServices;

/**
 * This is the base class for out concrete example classes. The concrete classes declare a public method for each of the web methods they can call.<br>
 * This base class reflectively gets these methods and asks the user to choose one.
 * 
 * @param <T> the web service type. For <em>WS_SINGLE_PACK</em> this would be {@link ISinglePackServices}, see {@link WsSinglePackExample}.
 */
public abstract class AbstractExample<T> {

    /**
     * Prints the available web methods and asks the user to choose one. The chosen method will then be called.
     */
    public final void invoke() {
        // Get all available web methods, sorted by name
        Method[] methods = getWSInterfaceClass().getMethods();
        Arrays.sort(methods, comparing(Method::getName));

        // ask the user to choose a method
        for (int i = 0; i < methods.length; i++) {
            System.out.println((i + 1) + ": " + methods[i].getName());
        }
        System.out.println("Enter index of web service method to try:");
        int methodIdx;
        try (Scanner scanner = new Scanner(System.in)) {
            while (true) {
                try {
                    methodIdx = Integer.parseInt(scanner.nextLine()) - 1;
                    if (0 <= methodIdx && methodIdx < methods.length) {
                        break;
                    }
                } catch (NumberFormatException ex) {}
                System.err.println("Please enter a number in the range [1-" + methods.length + "]");
            }

            System.out.println("Calling " + methods[methodIdx].getName());
            try {
                // reflectively call the chosen method
                getClass().getMethod(methods[methodIdx].getName()).invoke(this);
            } catch (InvocationTargetException ex) {
                throw new RuntimeException(ex.getCause());
            } catch (ReflectiveOperationException ex) {
                throw new RuntimeException(ex);
            }
        }
    }

    /**
     * Returns the WS port, which is a concrete implementation of the generated web service interface.
     * 
     * @return WS port.
     */
    protected final T getPort() {
        ClientProxyFactoryBean factory = new JaxWsProxyFactoryBean();
        factory.setAddress(Main.WS_PROPERTIES.getProperty(String.format("url.%s", getWsName())));
        factory.setBindingId(SOAPBinding.SOAP12HTTP_BINDING);

        LoggingOutInterceptor outInterceptor = new LoggingOutInterceptor(new PrintWriter(System.out));
        outInterceptor.setPrettyLogging(true);
        factory.getOutInterceptors().add(outInterceptor);

        LoggingInInterceptor inInterceptor = new LoggingInInterceptor(new PrintWriter(System.out));
        inInterceptor.setPrettyLogging(true);
        factory.getInInterceptors().add(inInterceptor);

        return factory.create(getWSInterfaceClass());
    }

    /**
     * Returns the class for the generated web service interface.
     * This is effectively <code>T.class</code>, if Java would allow that.
     * 
     * @return the class for the generated web service interface.
     */
    protected abstract Class<T> getWSInterfaceClass();

    /**
     * The name of the web service implemented in this example to present to the user.
     * 
     * @return the name of the web service implemented in this example to present to the user.
     */
    public abstract String getWsName();

}
