package de.arvato.systems.nmvs.example;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.Properties;
import java.util.Scanner;

public abstract class Main {

    /** Properties containing the service' endpoint URLs and user credentials. */
    public static final Properties WS_PROPERTIES;
    private static final Scanner   scanner           = new Scanner(System.in);
    public static final int        INTERFACE_VERSION = 3;

    static {
        WS_PROPERTIES = new Properties();
        try (BufferedReader br = new BufferedReader(new InputStreamReader(AbstractExample.class.getResourceAsStream("/ws.properties"), //
                StandardCharsets.UTF_8))) {
            WS_PROPERTIES.load(br);
        } catch (IOException ex) {
            throw new RuntimeException("Error reading ws.properties", ex);
        }
    }

    public static void main(String[] args) throws Exception {
        System.out.println(String.format("\nNMVS Client Examples Runner for interface version v%d\n", INTERFACE_VERSION));
        try {
            Class<?> clazz = Class.forName(String.format("de.arvato.systems.nmvs.example.v%1$d.MainV%1$d", INTERFACE_VERSION));
            final Main main = (Main) clazz.newInstance();
            main.invoke();
        } catch (ClassNotFoundException ex) {
            throw new IllegalArgumentException("Version number not supported", ex);
        }
    }

    // CAVEATS:
    // 1) Scanner is not threadsafe
    // 2) on closing the scanner instance the underlying stream is closed 
    // therefore we declare a shared instance and synchronize the method
    protected static synchronized int retrieveIntegerFromStdin(final String inputDescription, final int minValue, final int maxValue) {
        final String inputMessage = (maxValue == Integer.MAX_VALUE ? inputDescription + ": "
                : String.format("%s [%d - %d]: ", inputDescription, minValue, maxValue));
        int value = -1;
        do {
            System.out.println(inputMessage);
            do {
                try {
                    Thread.sleep(250L);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            } while (!scanner.hasNextLine());
            if (scanner.hasNextInt()) {
                value = scanner.nextInt();
                break;
            }
        } while (value > maxValue || value < minValue);
        System.out.println(String.format("[Received %d as input]", value));
        return value;
    }

    public Main() {
        super();
    }

    public abstract void invoke();

    protected void invoke(AbstractExample<?>[] wsExamples) {
        setUpClientCert();

        for (int i = 0; i < wsExamples.length; i++) {
            System.out.println((i + 1) + ": " + wsExamples[i].getWsName());
        }
        final int serviceIdx = retrieveIntegerFromStdin("Enter index of web service to try", 1, wsExamples.length);
        // invoke the chosen example. The example will print the available web methods and ask the user to choose one
        try {
            wsExamples[serviceIdx - 1].invoke();
        } catch (Exception ex) {
            ex.printStackTrace(System.err);
            System.err.println("Error calling web method. Please check the configuration in src/main/resources/ws.properties");
        }
    }

    /** Sets the <code>javax.net.ssl.keyStore</code> property to the path to our PKCS12 certificate. */
    private void setUpClientCert() {
        // Tell the SSL handshake mechanisms to use our client certificate
        System.setProperty("javax.net.ssl.keyStore", WS_PROPERTIES.getProperty("keystore.path"));
        System.setProperty("javax.net.ssl.keyStoreType", WS_PROPERTIES.getProperty("keystore.type"));
        System.setProperty("javax.net.ssl.keyStorePassword", WS_PROPERTIES.getProperty("keystore.password"));
    }

}
